/*************************************************************************
 *
 *  $RCSfile: soundhandler.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: as $ $Date: 2001/12/06 14:55:13 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#ifndef __FRAMEWORK_DISPATCH_SOUNDHANDLER_HXX_
#include <dispatch/soundhandler.hxx>
#endif

#ifndef __FRAMEWORK_CLASSES_ARGUMENTANALYZER_HXX_
#include <classes/argumentanalyzer.hxx>
#endif

#ifndef __FRAMEWORK_THREADHELP_TRANSACTIONGUARD_HXX_
#include <threadhelp/transactionguard.hxx>
#endif

#ifndef __FRAMEWORK_THREADHELP_WRITEGUARD_HXX_
#include <threadhelp/writeguard.hxx>
#endif

#ifndef __FRAMEWORK_THREADHELP_READGUARD_HXX_
#include <threadhelp/readguard.hxx>
#endif

#ifndef __FRAMEWORK_MACROS_DEBUG_HXX_
#include <macros/debug.hxx>
#endif

#ifndef __FRAMEWORK_SERVICES_H_
#include <services.h>
#endif

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_IO_XINPUTSTREAM_HPP_
#include <com/sun/star/io/XInputStream.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _RTL_USTRBUF_HXX_
#include <rtl/ustrbuf.hxx>
#endif

#ifndef _SV_SALSTYPE_HXX
#include <vcl/salstype.hxx>
#endif

//_________________________________________________________________________________________________________________
//	namespace
//_________________________________________________________________________________________________________________

namespace framework{

//_________________________________________________________________________________________________________________
//	non exported const
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	non exported definitions
//_________________________________________________________________________________________________________________

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

//*****************************************************************************************************************
//  XInterface, XTypeProvider, XServiceInfo
//*****************************************************************************************************************
DEFINE_XINTERFACE_4                 (   SoundHandler                                                 ,
                                        OWeakObject                                                  ,
                                        DIRECT_INTERFACE( css::lang::XTypeProvider                  ),
                                        DIRECT_INTERFACE( css::lang::XServiceInfo                   ),
                                        DIRECT_INTERFACE( css::frame::XDispatch                     ),
                                        DIRECT_INTERFACE( css::document::XExtendedFilterDetection   )
                                    )

DEFINE_XTYPEPROVIDER_4              (   SoundHandler                                                ,
                                        css::lang::XTypeProvider                                    ,
                                        css::lang::XServiceInfo                                     ,
                                        css::frame::XDispatch                                       ,
                                        css::document::XExtendedFilterDetection
                                    )

DEFINE_XSERVICEINFO_MULTISERVICE    (   SoundHandler                                                ,
                                        ::cppu::OWeakObject                                         ,
                                        SERVICENAME_SOUNDHANDLER                                    ,
                                        IMPLEMENTATIONNAME_SOUNDHANDLER
									)

DEFINE_INIT_SERVICE                 (   SoundHandler,
                                        {
                                        }
                                    )

/*-************************************************************************************************************//**
    @short      standard ctor
    @descr      These initialize a new instance of this class with needed informations for work.

    @seealso    using at owner

    @param      "xFactory", reference to service manager for creation of new services
    @return     -

    @onerror    Show an assertion and do nothing else.
    @threadsafe yes
*//*-*************************************************************************************************************/
SoundHandler::SoundHandler( const css::uno::Reference< css::lang::XMultiServiceFactory >& xFactory )
		//	Init baseclasses first
        :   ThreadHelpBase      (          )
        ,   ::cppu::OWeakObject (          )
        // Init member
        ,   m_xFactory          ( xFactory )
        ,   m_xSelfHold         ( NULL     )
        ,   m_aPlayer           (          )
        ,   m_nRequests         ( 0        )
{
	// Safe impossible cases
    LOG_ASSERT2( impldbg_checkParameter_SoundHandler( xFactory ), "SoundHandler::SoundHandler()", "Invalid parameter detected!" )

    m_aPlayer.SetNotifyHdl( LINK( this, SoundHandler, implts_PlayerNotify ) );
}

/*-************************************************************************************************************//**
    @short      standard dtor
    @descr      -

    @seealso    -

    @param      -
    @return     -

    @onerror    -
    @threadsafe -
*//*-*************************************************************************************************************/
SoundHandler::~SoundHandler()
{
}

/*-************************************************************************************************************//**
    @interface  ::com::sun::star::frame::XDispatch

    @short      try to load audio file
    @descr      This method try to load given audio file by URL and play it. We use vcl/Sound class to do that.
                Playing of sound is asynchron everytime. So we create temp. member m_pPlayer by using
                ref count mechanism m_nRequests.

    @attention  We must hold us alive by ourself ... because we use async. vcl sound player ... but playing is started
                in async interface call "dispatch()" too. And caller forget us imediatly. But then our uno ref count
                will decreased to 0 and will die. The only solution is to use own reference to our implementation.
                But we do it for realy started jobs only and release it during call back of vcl.

    @seealso    class vcl/Sound
    @seealso    method implts_PlayerNotify()

    @param      "aURL"      , URL to dispatch.
    @param      "lArguments", list of optional arguments.
    @return     -

    @onerror    We do nothing.
    @threadsafe yes
*//*-*************************************************************************************************************/
void SAL_CALL SoundHandler::dispatch( const css::util::URL&                                  aURL       ,
                                      const css::uno::Sequence< css::beans::PropertyValue >& lArguments ) throw( css::uno::RuntimeException )
{
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */
    // Safe impossible cases ... check for wrong call parameters
    LOG_ASSERT2( impldbg_checkParameter_dispatch( aURL, lArguments ), "SoundHandler::dispatch()", "Invalid parameter detected." )

    ResetableGuard aLock( m_aLock );

    // If player currently used for other dispatch() requests ...
    // cancel it by calling stop()!
    if( m_aPlayer.IsPlaying() == sal_True )
        m_aPlayer.Stop();

    // Try to initialize player.
    if( m_aPlayer.SetSoundName( aURL.Complete ) == sal_True )
    {
        // OK- we can start async playing ...
        // Count this request and initialize self-holder against dieing by uno ref count ...
        if( m_nRequests==0 )
            m_xSelfHold = static_cast< ::cppu::OWeakObject* >(this);
        ++m_nRequests;
        m_aPlayer.Play();
    }
}

/*-************************************************************************************************************//**
    @interface  ::com::sun::star::document::XExtendedFilterDetection

    @short      try to detect file (given as argument included in "lDescriptor")
    @descr      We try to detect, if given file could be handled by this class and is a well known one.
                If it is - we return right internal type name - otherwise we return nothing!
                So call can search for another detect service and ask him too.

    @attention  a) We don't need any mutex here ... because we don't use any member!
                b) Dont' use internal player instance "m_pPlayer" to detect given sound file!
                   It's not neccessary to do that ... and we can use temp. variable to do the same.
                   This way is easy - we don't must synchronize it with currently played sounds!
                   Another reason to do so ... We are a listener on our internal ma_Player object.
                   If you would call "IsSoundFile()" on this instance, he would call us back and
                   we make some uneccssary things ...

    @seealso    -

    @param      "lDescriptor", description of file to detect
    @return     Internal type name which match this file ... or nothing if it is unknown.

    @onerror    We return nothing.
    @threadsafe yes
*//*-*************************************************************************************************************/
::rtl::OUString SAL_CALL SoundHandler::detect( css::uno::Sequence< css::beans::PropertyValue >& lDescriptor ) throw( css::uno::RuntimeException )
{
    /* UNSAFE AREA --------------------------------------------------------------------------------------------- */
    // Safe impossible cases ... check for wrong call parameters
    LOG_ASSERT2( impldbg_checkParameter_detect( lDescriptor ), "SoundHandler::detect()", "Invalid parameter detected." )

    // Our default is "nothing". So we can return it, if detection failed or fily type is realy unknown.
    ::rtl::OUString sTypeName;

    // Analyze given descriptor to find filename or input stream or ...
    ArgumentAnalyzer                                aAnalyzer( lDescriptor );
    ::rtl::OUString                                 sURL                    ;
    css::uno::Reference< css::io::XInputStream >    xStream                 ;
    sal_Bool                                        bURLExist               ;
    sal_Bool                                        bStreamExist            ;
    Sound                                           aDetector               ;

    bURLExist    = aAnalyzer.getArgument( E_URL        , sURL    );
    bStreamExist = aAnalyzer.getArgument( E_INPUTSTREAM, xStream );

    // A stream should be used if her exist .. otherwise we must use URL.
    // So we avoid against multiple openining of file!
    if( bStreamExist == sal_True )
    {
        LOG_WARNING( "SoundHandler::detect()", "Detection by using stream not implemented yet!" )
    }
    else
    if( bURLExist == sal_True )
    {
        if( aDetector.IsSoundFile( sURL ) == sal_True )
        {
            // "IsSoundFile" idffer between different "wav" and "au" file versions ...
            // couldn't return this information ... because: He use the OS to detect it!
            // I think we can the following ones:
            //  a) look for given extension of url to map our type decision HARD CODED!!!
            //  b) return preferred type every time... it's easy :-)
            sTypeName = DECLARE_ASCII("wav_Wave_Audio_File");
            aAnalyzer.setArgument( E_TYPENAME, sTypeName );
        }
    }

    // Return our decision.
    return sTypeName;
}

/*-************************************************************************************************************//**
    @short      call back of sound player
    @descr      Our player call us back to give us some informations.
                These information aren't realy neccessary for us ( we don't support listener !)
                but we must decide, if we must let us die by ourself ...

    @attention  We use m_xSelfHold to let us die ... but we must live till real finishing of this method too!!!
                So we SHOULD use another "self-holder" temp. to provide that ...

    @seealso    method dispatch()

    @param      -
    @return     0 everytime ... it doesnt matter for us.

    @onerror    -
    @threadsafe yes
*//*-*************************************************************************************************************/
IMPL_LINK( SoundHandler, implts_PlayerNotify, void*, pVoid )
{
    css::uno::Reference< css::uno::XInterface > xOperationHold = static_cast< ::cppu::OWeakObject* >(this);

    /* SAFE AREA ----------------------------------------------------------------------------------------------- */
    ResetableGuard aLock( m_aLock );
    --m_nRequests;
    if( m_nRequests==0 )
        m_xSelfHold = NULL;

    return 0;
}

/*-************************************************************************************************************//**
    @short      debug-methods to check incoming parameter of our interface mehods
    @descr      The following methods are used to check incoming parameters
                of this class. The return value is used directly for an ASSERT(...).
                So we control rightt using of this class an guarantee correct results of operations.

    @seealso    ASSERTs in implementation!

    @param      References to checking variables
    @return     false on invalid parameter<BR>
                true  otherway

    @onerror    We return false.
    @threadsafe They mustn't be threadsafe - because they are static and work on given parameters only!
*//*-*************************************************************************************************************/
#ifdef ENABLE_ASSERTIONS

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_SoundHandler( const css::uno::Reference< css::lang::XMultiServiceFactory >& xFactory )
{
    return   (
                ( &xFactory      == NULL      ) ||
                (  xFactory.is() == sal_False )
             );
}

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_dispatch( const css::util::URL&                                  aURL       ,
                                                        const css::uno::Sequence< css::beans::PropertyValue >& lArguments )
{
    return(
            ( &aURL                     ==  NULL    )   ||
            ( aURL.Complete.getLength() <   1       )   ||
            ( &lArguments               ==  NULL    )
          );
}

//*****************************************************************************************************************
sal_Bool SoundHandler::impldbg_checkParameter_detect( css::uno::Sequence< css::beans::PropertyValue >& lDescriptor )
{
    return   (
                ( &lDescriptor            == NULL )    ||
                ( lDescriptor.getLength() <  1    )
             );
}

#endif	//	#ifdef ENABLE_ASSERTIONS

}		//	namespace framework
